@@ -6,8 +6,9 @@
 library(plgp)
 library(tgp)
 
-## close down old graphics windows
+## close down old graphics windows and clear session
 graphics.off()
+rm(list=ls())
 
 ## set up 1-d data
 f1d <- function(x, sd=0.15)
@@ -47,7 +48,7 @@ start <- 20
 end <- 50
 
 ## do the particle learning
-out <- PL(data=data.GP.improv, ## adaptive design PL via IECI
+out <- PL(dstream=data.GP.improv, ## adaptive design PL via IECI
           start=start, end=end,
           init=draw.GP,  ## init with Metropolis-Hastings
           lpredprob=lpredprob.GP, propagate=propagate.GP,
@@ -55,16 +56,16 @@ out <- PL(data=data.GP.improv, ## adaptive design PL via IECI
 
 ## sample from the particle posterior predictive distribution
 XXs <- rectscale(XX, rect)
-outp <- papply(XX=XXs, fun=pred.GP, Y=pall$Y, quants=TRUE, prior=prior)
+outp <- papply(XX=XXs, fun=pred.GP, Y=PL.env$pall$Y, quants=TRUE, prior=prior)
 
 ## unscale the data locations
-X <- rectunscale(pall$X, rect)
+X <- rectunscale(PL.env$pall$X, rect)
 
 ## set up to make two plots
 par(mfrow=c(1,2))
 
 ## plot the individual lines of the predictive distribution
-plot(X,pall$Y, main="predictive: each particle", xlab="x", ylab="y")
+plot(X,PL.env$pall$Y, main="predictive: each particle", xlab="x", ylab="y")
 for(i in 1:length(outp)) {
   lines(XX, outp[[i]]$m)
   lines(XX, outp[[i]]$q1, col=2, lty=2)
@@ -86,7 +87,7 @@ q1 <- q1 / length(outp)
 q2 <- q2 / length(outp)
 
 ## plot the summary stats of the predictive distribution
-plot(X, pall$Y, main="predictive surface",
+plot(X, PL.env$pall$Y, main="predictive surface",
      ylim=range(c(q1,q2)), xlab="x", ylab="y")
 lines(XX, m, lwd=2)
 lines(XX, q1, col=2, lty=2, lwd=2)
@@ -100,7 +101,7 @@ par(mfrow=c(1,2)) ## two plots
 plot(X, xlab="t")
 abline(v=start+1, lwd=2, col=2, lty=2)
 abline(h=c(2,4), lwd=2, col=3, lty=3)
-points((start+1):end, psave$xstar, col=4, pch=18)
+points((start+1):end, PL.env$psave$xstar, col=4, pch=18)
 ## plot the max log IECI 
-plot((start+1):end, psave$max.as, type="l", xlab="t",
+plot((start+1):end, PL.env$psave$max.as, type="l", xlab="t",
      ylab="max log IECI", main="progress meter")
